import unittest
from charpy.chartjs.chart import *


class TestChart(unittest.TestCase):
    def setUp(self):
        self.chart = Chart('bar')

    def test_500x_wrong_chart_type(self):
        self.assertRaises(TypeError, Chart, 'not a chart type')

    def test_501_title_not_setted_return_None(self):
        self.assertEqual(self.chart.title, None)

    def test_502_title_can_be_setted(self):
        self.chart.title = "test"
        self.assertEqual(self.chart.title, "test")
        self.assertTrue(self.chart._title_display)

    def test_503_title_is_a_chart_attribute(self):
        self.assertTrue(hasattr(self.chart, 'title'))

    def test_504x_wrong_None_title(self):
        with self.assertRaises(ValueError):
            self.chart.title = None

    def test_505x_wrong_empty_title(self):
        with self.assertRaises(ValueError):
            self.chart.title = ''

    def test_510_toggle_legend(self):
        self.assertEqual(self.chart._legend_display, JS_CONVERT[False])
        self.chart.toggle_legend_display()
        self.assertEqual(self.chart._legend_display, JS_CONVERT[True])

    def test_511x_chart_with_wrong_legend_display_value(self):
        self.assertRaises(ValueError, Chart, 'bar', legend_display="random_stuff")

    def test_520_add_a_simple_dataset(self):
        self.chart.add_simple_dataset(data_label='numbers', data=[1, 2, 3])
        self.chart.datasets


class TestDataset(unittest.TestCase):
    def setUp(self):
        self.d = Dataset([1, 2, 3], 'numbers')

    def test_600_tojson_dataset(self):
        d_json = '{"backgroundColor": "royalBlue", "borderColor": "white", "data": [1, 2, 3], "label": "numbers"}'
        self.assertEqual(self.d.to_json(), d_json)

    def test_601_dataset_well_created(self):
        self.assertEqual(self.d.data, [1, 2, 3])
        self.assertEqual(self.d.label, 'numbers')
        self.assertEqual(self.d.borderColor, "white")
        self.assertEqual(self.d.backgroundColor, "royalBlue")

    def test_610_set_label(self):
        label = "the label"
        self.d.set_label(label)
        self.assertEqual(self.d.label, label)

    def test_611x_wrong_type_label(self):
        self.assertRaises(TypeError, self.d.set_label, 1)

    def test_620_set_data(self):
        data = [4, 5, 6]
        self.d.set_data(data)
        self.assertEqual(self.d.data, data)

    def test_621x_wrong_type_numeric_data(self):
        self.assertRaises(TypeError, self.d.set_data, 1)

    def test_622x_wrong_type_string_data(self):
        self.assertRaises(TypeError, self.d.set_data, 'abc')

    def test_623x_wrong_type_tuple_or_dict_data(self):
        tupo = (1, 2)
        self.assertRaises(TypeError, self.d.set_data, tupo)

    def test_624x_wrong_type_dict_data(self):
        dico = {'x': 1, 'y': 2}
        self.assertRaises(TypeError, self.d.set_data, dico)


if __name__ == "__main__":
    unittest.main()



import os
from charpy.cruncher import Orc
from charpy import MOCK_PATH
from charpy.chartjs import chart
from flask import Blueprint, render_template

bp = Blueprint('charpy', __name__, template_folder='templates')


@bp.route('/')
@bp.route('/chart_demo/default/')
def chart_demo():
    demo = Orc(os.path.join(MOCK_PATH, "pcbanking.csv"))
    return render_template('view/chart_demo.html', values=demo.df['value'], labels=demo.df['label'], type='pie')


@bp.route('/chart_demo/static/')
def chart_static_demo():
    labels = ["January", "February", "March", "April", "May", "June", "July", "August"]
    values = [10, 9, 8, 7, 6, 4, 7, 8]
    colors = ["#F7464A", "#46BFBD", "#FDB45C", "#FEDCBA", "#ABCDEF", "#DDDDDD", "#ABCABC", "#BCAFED"]
    return render_template('view/chart_static.html', version="1.0.1", values=values, labels=labels, set=zip(values, labels, colors))


@bp.route('/chart_demo/charpy/')
def chart_chartjs():
    demo = Orc(os.path.join(MOCK_PATH, "pcbanking.csv"))
    chart_sdemo = chart.Chart('bar')
    chart_sdemo.title = "Test chart from template"
    chart_sdemo.add_dataset('value', demo.df['value'].tolist())
    chart_sdemo.labels = demo.df['label'].tolist()

    return chart_sdemo.render_flask('view/chartjs_default.html')


@bp.route("/chart_demo/hello/<string:name>/")
def hello(name):
    return "hello " + name + "!"



import json
from flask import render_template
from charpy.chartjs import check_chart_type, JS_CONVERT, PYTHON_CONVERT


class Chart(object):
    def __init__(self, chart_type, legend_display=False, canvas_id='chart'):
        self.canvas_id = canvas_id
        self.chart_type = check_chart_type(chart_type)
        self.labels = []
        self.datasets = ''

        try:
            self._legend_display = JS_CONVERT[legend_display]
        except KeyError:
            raise ValueError("legend_display, can only accept True or False - input: '{}'".format(legend_display))

        self._title = ''
        self._title_display = JS_CONVERT[False]

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, title):
        """
        Set _title.display to 'true' the javascript value for True
        When the title is set.

        """
        if title is not None and title != '':
            self._title_display = JS_CONVERT[True]
            self._title = title
        else:
            raise ValueError("Error: Can't set '{}' - an empty String or None to title'".format(title))

    @title.getter
    def title(self):
        """ Return the title if it is displayed """
        if PYTHON_CONVERT[self._title_display]:
            return self._title
        else:
            return None

    def toggle_legend_display(self):
        """
        toggle the display of the legend between 'true' or 'false'
        which are the javascript equivalent for True and False.

        Then return the legend display value into python Boolean.

        """
        self._legend_display = JS_CONVERT[not PYTHON_CONVERT[self._legend_display]]
        return PYTHON_CONVERT[self._legend_display]

    def add_simple_dataset(self, data_label, data):
        """


        :param data_label: label of the data used for the legend
        :param data:
        :return:
        """
        dataset_json = json.dumps({"label": data_label, "data": data})
        self.add_dataset(dataset_json)

    def add_dataset(self, dataset_json):
        """
        Add a dataset from a dataset object

        :param dataset_json:
        :return:
        """
        self.datasets += dataset_json + ","

    def render_html(self):
        """
        Generate an html page of the chart.

        :return:
        """
        return 0

    def render_flask(self, flask_template):
        """
        Use flask and the jinja2 templating system to render the chart

        :param flask_template:
        :return: rendered page
        """
        return render_template(flask_template,
                               script_local=True,
                               id=self.canvas_id,
                               type=self.chart_type,
                               labels=self.labels,
                               datasets=self.datasets,
                               legend_display=self._legend_display,
                               title_display=self._title_display,
                               title=self._title
                               )


class Dataset(object):
    def __init__(self, data, label=''):
        self.label = label
        self.data = data
        self.backgroundColor = "royalBlue"
        self.borderColor = "white"

    def set_label(self, label):
        if type(label) == str:
            self.label = label
        else:
            raise TypeError("Error: The label should be a string - input '{}'".format(label))

    def set_data(self, data):
        if type(data) == list:
            self.data = data
        else:
            raise TypeError("Error: data can't be a string - '{}'".format(data))

    def to_json(self):

        return json.dumps(self.__dict__, sort_keys=True, ensure_ascii=False)


class LineDataset(Dataset):  # pragma: no cover

    def __init__(self, data, label, fill=False, showLine=True):
        super().__init__(data, label)
        self.fill = fill
        self.showLine = showLine



from flask import Blueprint, render_template

bp = Blueprint('default', __name__, template_folder='templates')


@bp.app_errorhandler(404)
def page_not_found(error):
    return render_template('layout/error.html', color="#2B547E", error=404, message_1="THE PAGE", message_2="WAS NOT FOUND"), 404


@bp.app_errorhandler(501)
def page_not_found(error):
    return render_template('layout/error.html', color="#006600", error=501, message_1="FEATURE", message_2="NOT IMPLEMENTED"), 501



__chartjs_version__ = "2.7.2"

SCRIPT = "<script src='https://cdnjs.cloudflare.com/ajax/libs/Chart.js/{}/Chart.min.js'>" \
            "</script>".format(__chartjs_version__)
CANVAS = "<canvas id='{}' width='600' height='400'></canvas>"
HTML = '''
        <!DOCTYPE html>
        <html>
            <head>
                <meta charset="UTF-8">
                <title>{}</title>
                {}
            </head>
            <body>
                <div id="content">
                    {}
                </div>
                <footer>
                </footer>
            </body>
        </html>
        '''
CHARTJS = '''
            <script>
                new Chart(document.getElementById("{}"), {
                    type: '{}',
                    data: {
                      labels: {},
                      datasets: [{}]
                    },
                    options: {
                      legend: { display: {} },
                      title: {
                        display: {},
                        text: '{}'
                      }
                    }
                });
            
             </script>
            '''

JS_CONVERT = {True: 'true', False: 'false'}
PYTHON_CONVERT = {'true': True, 'false': False}

LINE = "line"
BAR = "bar"
HORIZONTAL_BAR = "horizontalBar"
RADAR = "radar"
POLAR_AREA = "polarArea"
DOUGHNUT = "doughnut"
PIE = "pie"
BUBBLE = "bubble"
SCATTER = "scatter"

TYPE = {
    LINE,
    BAR,
    HORIZONTAL_BAR,
    RADAR,
    POLAR_AREA,
    DOUGHNUT,
    PIE,
    BUBBLE,
    SCATTER
}


def is_chartjs_type(chart_type):
    """
    Return True if the chart type is handled by chartjs

    :param chart_type:
    :return: True or False
    """
    if chart_type in TYPE:
        return True
    else:
        return False


def check_chart_type(chart_type):
    if is_chartjs_type(chart_type):
        return chart_type
    else:
        raise TypeError("Error: The type '{}' is not a handled chartjs type".format(chart_type))


import unittest
from charpy.utils import *


class TestConverter(unittest.TestCase):

    def test_900_readme_md_to_rst_not_empty(self):
        """ check that readme to rst generate a file not empty """
        self.assertTrue(readme_md_to_rst(), True)

    def test_901_readme_md_converted_to_rst(self):
        """ If it is really converted the first character should not be a # """
        self.assertFalse(readme_md_to_rst()[1:] == "#")

    def test_902_get_readme_rst(self):
        """ check if you can retrieve the rst file and that a file is not created """
        result = get_readme_rst()
        self.assertFalse(result == "")

    def test_903_reate_readme_rst(self):
        """ check that readme to rst generate a file not empty """
        os.remove(README_RST_PATH)
        create_readme_rst()
        self.assertTrue(os.path.isfile(README_RST_PATH))
        self.assertTrue(os.path.getsize(README_RST_PATH) > 0)


if __name__ == "__main__":
    unittest.main()



import os
from charpy.cruncher import Orc
from charpy import MOCK_PATH
from charpy.chartjs import chart, is_chartjs_type
from flask import Blueprint, abort

bp = Blueprint('chartjs', __name__, template_folder='templates')


@bp.route('/chart/<string:chartype>/', methods=['GET'])
def chart_chartjs(chartype):

    if is_chartjs_type(chartype):
        chart_sdemo = chart.Chart(chartype)
        chart_sdemo.title = "Test chart from template"
        demo = Orc(os.path.join(MOCK_PATH, "pcbanking.csv"))
        chart_sdemo.add_simple_dataset('value', demo.df['value'].tolist())
        chart_sdemo.add_simple_dataset('test', demo.df['value'].tolist())
        chart_sdemo.labels = demo.df['label'].tolist()
        return chart_sdemo.render_flask('view/chartjs_default.html')
    else:
        abort(501)



import math


def color_based_on_sign(data, color_positive="green", color_negative="red"):
    """

    :param data:
    :param color_positive:
    :param color_negative:
    :return:
    """
    color_associated = color_based_on_operation(data, is_positive(), color_positive,
                                                color_negative)

    return color_associated


def color_based_on_value(data, matching_value, color_value="blue", color_not_value="lightgray"):
    color_associated = color_based_on_operation(data, is_equal(matching_value), color_value,
                                                color_not_value)

    return color_associated


def color_based_on_operation(data, matching_criteria, matching_color, not_matching_color):
    """

    :param data:
    :param matching_criteria:
    :param matching_color:
    :param not_matching_color:
    :return:
    """
    color_associated = list(map(lambda x: matching_color if matching_criteria(x) else not_matching_color, data))

    return color_associated


def is_positive():
    return lambda x: True if x >= 0 else False


def is_equal(matching_value):
    return lambda x: True if x == matching_value else False


def color_rainbow(data):
    rainbow = []
    size = len(data)
    for e in range(size):
        red = sin_to_hex(e, 0, size)
        blue = sin_to_hex(e, 2 * math.pi / 3, size)
        green = sin_to_hex(e, 4 * math.pi / 3, size)
        rainbow.append("#" + red + green + blue)

    return rainbow


def sin_to_hex(x, theta, number_of_taint):
    sin_value = math.sin(2 * math.pi * x / number_of_taint + theta)
    rgb_color = math.floor(sin_value * 127) + 128
    hex_color = format(rgb_color, "x")

    return hex_color if len(hex_color) == 2 else "0" + hex_color


if __name__ == "__main__":  # pragma: no cover
    data_array = [1, -1, 1, -1]
    print(color_based_on_sign(data_array))
    print(color_based_on_value(data_array, matching_value=1))

    print(is_positive()(1))
    print(is_equal(2)(1))

    print(sin_to_hex(0, 0, 20))
    print(color_rainbow(data_array))



import unittest
from charpy.chartjs.color import *


class TestColor(unittest.TestCase):
    def setUp(self):
        self.d = [-1, -2, 0, 1, 5]

    def test_is_positive_return_function(self):
        self.assertEqual(str(type(is_positive())), "<class 'function'>")

    def test_is_postive_true_positive(self):
        self.assertTrue(is_positive()(1))

    def test_is_postive_false_on_negative(self):
        self.assertFalse(is_positive()(-1))

    def test_is_postive_on_zero(self):
        self.assertTrue(is_positive()(0))

    def test_wrong_is_postive_letter(self):
        self.assertRaises(TypeError, is_positive(), "letter")

    def test_is_equal_return_function(self):
        self.assertEqual(str(type(is_equal(0))), "<class 'function'>")

    def test_wrong_is_equal_empty(self):
        self.assertRaises(TypeError, is_equal, (), 1)

    def test_is_equal_number(self):
        self.assertTrue(is_equal(1)(1))

    def test_is_not_equal_number(self):
        self.assertFalse(is_equal(1)(2))

    def test_is_equal_letter(self):
        self.assertTrue(is_equal("letter")("letter"))

    def test_is_not_equal_letter(self):
        self.assertFalse(is_equal("other")("letter"))

    def test_angle_to_hex_color(self):
        self.assertEqual(sin_to_hex(0, 0, 5), '80')
        self.assertEqual(sin_to_hex(0, 2 * math.pi / 3, 5), 'ed')
        self.assertEqual(sin_to_hex(0, 4 * math.pi / 3, 5), '12')

    def test_color_rainbow(self):
        rainbow_5 = ['#8012ed', '#f82165', '#cab301', '#35fe4c', '#079ade']
        self.assertEqual(color_rainbow(self.d), rainbow_5)

    def test_color_rainbow_empty(self):
        self.assertEqual(color_rainbow([]), [])

    def test_color_based_on_sign(self):
        output = ['red', 'red', 'green', 'green', 'green']
        self.assertEqual(color_based_on_sign(self.d), output)

    def test_color_based_on_value(self):
        output = ['lightgray', 'lightgray', 'lightgray', 'blue', 'lightgray']
        self.assertEqual(color_based_on_value(self.d, matching_value=1), output)


if __name__ == "__main__":
    unittest.main()



from flask import Blueprint, abort, Response, render_template
import os
from charpy.cruncher import Orc
from charpy import MOCK_PATH

bp = Blueprint('dataframe', __name__, template_folder='templates')


@bp.route('/dataframe/')
def dataframe_json_all():
    demo = Orc(os.path.join(MOCK_PATH, 'pcbanking.csv'))

    response = Response(
        response=demo.df.to_json(),
        status=200,
        mimetype='application/json'
    )

    return response


@bp.route('/dataframe/html/')
def dataframe_html():
    demo = Orc(os.path.join(MOCK_PATH, 'pcbanking.csv'))
    demo.format_column_list('label')
    demo.create_from_list_column('label')
    demo.create_from_date_column('date', formatting='day')
    demo.create_from_date_column('date', formatting='month')
    demo.create_from_date_column('date', formatting='year')

    return render_template(os.path.join('view', 'dataframe.html'), body=demo.df.to_html())


@bp.route('/dataframe/<string:column>/', methods=['GET'])
def dataframe_json_column(column):
    demo = Orc(os.path.join(MOCK_PATH, "pcbanking.csv"))
    try:
        response = Response(
            response=demo.get_json_data(column),
            status=200,
            mimetype='application/json'
        )
        return response
    except KeyError:
        abort(404)



# -*- coding: utf-8 -*-
"""
Use export FLASK_APP="charpy.factory:create_app()"


"""
from flask import Flask
from charpy import TEST_DATABASE_URI
from werkzeug.utils import find_modules, import_string


def create_app(config=None, debug=False):
    app = Flask(__name__)
    app.debug = debug

    app.config.update(dict(
        DATABASE=TEST_DATABASE_URI,
        DEBUG=True,
        SECRET_KEY='development key',
        USERNAME='admin',
        PASSWORD='default'
    ))
    app.config.update(config or {})
    app.config.from_envvar('FLASKR_SETTINGS', silent=True)

    register_blueprints(app)

    return app


def register_blueprints(app):
    """Automagically register all blueprint named bp in packages

    Check the argument 'bp' in all the modules in the folder inserted in findmodules
    then register all blueprints in the app
    """
    for name in find_modules('charpy.flask.blueprints', recursive=True):
        mod = import_string(name)

        if hasattr(mod, 'bp'):
            app.register_blueprint(mod.bp)
    return None


if __name__ == "__main__":  # pragma: no cover
    app = create_app(debug=True)
    # print(app.blueprints)

    app.run()




import os
import unittest
import datetime
from charpy import MOCK_PATH, SQL_PATH, TEST_DATABASE_URI
from charpy.cruncher import Orc, check_path
from tests import *


class TestConverter(unittest.TestCase):

    def setUp(self):
        self.file = Orc(os.path.join(MOCK_PATH, "demo.csv"), sep=',', header=0)
        self.semicolon = Orc(os.path.join(MOCK_PATH, "demo-semicolon.csv"), header=0)

    def test_000_is_csv_delimiter_agnostic_comma(self):
        """ The csv file can be opened however the delimiter """
        preset = self.file.df.columns.tolist()
        comma = Orc(os.path.join(MOCK_PATH, "demo.csv"), header=0).df.columns.tolist()
        self.assertEqual(preset, comma)

    def test_001_is_csv_delimiter_agnostic_semicolon(self):
        """ The csv file can be opened however the delimiter """
        preset = self.file.df.columns.tolist()
        semicolon = self.semicolon.df.columns.tolist()
        self.assertEqual(preset, semicolon)

    def test_002_is_default_header(self):
        """ When no header parameters, the default one is taken """
        comma = Orc(os.path.join(MOCK_PATH, "demo.csv")).df
        self.assertFalse(comma.columns.values.all() == self.file.df.columns.values.all())
        self.assertTrue(list(comma) == Orc.DEFAULT_NAMES)

    def test_003_open_from_directory(self):
        """ Path can be a directory """
        self.file = Orc(os.path.join(MOCK_PATH))
        self.assertTrue(self.file.df.shape[0] == 33)

    def test_004_get_colum_values_as_list(self):
        """ Make sure we get the column values from a csv file """
        self.assertEqual(TEST_DATA_COLUMN, self.file.get_column_values())

    def test_010x_is_wrong_path_empty(self):
        """ Wrong path type give typeError """
        print('\n\nWrong path input raise exception')
        self.assertRaises(TypeError, check_path([]))

    def test_011x_is_wrong_path_numeric(self):
        """ Wrong path type give typeError """
        self.assertRaises(TypeError, check_path(2))

    def test_120_is_date_formatted_default(self):
        """ The date should be formatted in month """
        self.file.format_column_date(0)
        self.semicolon.format_column_date('Month')
        self.assertEqual(self.file.df['Month'].all(), self.semicolon.df[self.semicolon.df.columns[0]].all())

    def test_121_is_date_formatted_month(self):
        """ Change data format to default then back to Month and make sure it is still ok """
        self.file.format_column_date(0)
        self.file.format_column_date(0, formatting='%B')
        self.assertEqual(self.file.df['Month'].tolist(), TEST_DATA_COLUMN[0])

    def test_122_is_new_date_column_created_day(self):
        """ Test if the new date column is created using the right format """
        self.file.create_from_date_column(formatting="day", input_date_column=0, output_column_name="date-day")
        self.assertTrue(self.file.df["date-day"].str.contains('day').all())
        self.assertTrue(len(self.file.df.columns) == 4)

    def test_123_is_new_date_column_created_month(self):
        """ Test if the new date column is created using the right format """
        self.file.create_from_date_column(formatting="month", input_date_column=0, output_column_name="date-month")
        self.assertTrue(list(self.file.df["date-month"]), TEST_DATA_COLUMN[0])
        self.assertTrue(len(self.file.df.columns) == 4)

    def test_124_is_new_date_column_created_year(self):
        """ Test if the new date column is created using the right format """
        self.file.create_from_date_column(formatting="year", input_date_column=0, output_column_name="date-year")
        self.assertTrue(self.file.df["date-year"].str.contains(str(datetime.date.today().year)).all())
        self.assertTrue(len(self.file.df.columns) == 4)

    def test_130_is_list_formatted(self):
        """ test column if formatted to list """
        self.file.format_column_list(2, regex=r'#')
        self.assertTrue(len(self.file.df['Colors'][0]) == 2 and type(self.file.df['Colors'][0]) == list)

    def test_131_are_new_column_created(self):
        """ Make sure columns are created """
        self.file.format_column_list(2, regex=r'#')
        self.assertTrue(len(self.file.df.columns) == 3)
        self.file.create_from_list_column(2, names=['hashtag', 'hexa'])
        self.assertTrue(len(self.file.df.columns) == 5)

    def test_132_are_more_column_created(self):
        """ Make sure the right amount of column is created eventhough there's not enough data """
        self.file.format_column_list(2, regex=r'#')
        self.file.create_from_list_column(2, names=['hashtag', 'hexa', 'stuff'])
        self.assertTrue(len(self.file.df.columns) == 6)

    def test_140x_wrong_column_string(self):
        """ Raise an error is a wrong column identifier is put """
        print('\n\nWrong column input raise exception')
        self.assertRaises(ValueError, self.file.format_column_date('test'))

    def test_141x_wrong_column_negative_number(self):
        """ Raise an error is a wrong column identifier is put """
        self.assertRaises(TypeError, self.file.format_column_date(-1))

    def test_142x_wrong_column_number(self):
        """ Raise an error is a wrong column identifier is put """
        self.assertRaises(IndexError, self.file.format_column_date(100))

    def test_143x_wrong_date_format_number(self):
        """ Raise an error is a wrong column identifier is put """
        print('\n\nWrong format input raise exception')
        self.assertRaises(TypeError, self.file.format_column_date(0, formatting=1))

    def test_144x_wrong_date_format_string(self):
        """ Raise an error is a wrong column identifier is put """
        self.assertRaises(ValueError, self.file.format_column_date(0, formatting="not a format"))

    def test_145x_wrong_not_list_column(self):
        """ Raise an error when the column has no list object for creation """
        print('\n\nWrong not list column raise exception')
        self.assertRaises(TypeError, self.file.create_from_list_column(2, names=['hashtag', 'hexa']))

    def test_150_is_database_created(self):
        """ Check if the db is created """
        test_db_path = TEST_DATABASE_URI
        self.file.create_sql_db(db_path=test_db_path)
        self.assertTrue(os.path.exists(test_db_path))


if __name__ == "__main__":
    unittest.main()



from __future__ import print_function

import sys
import collections
import importlib
import json
import pkgutil
import warnings
from functools import wraps

import plotly
import dash_renderer
import flask
from flask import Flask, Response
from flask_compress import Compress

from .dependencies import Event, Input, Output, State
from .resources import Scripts, Css
from .development.base_component import Component
from . import exceptions
from ._utils import AttributeDict as _AttributeDict


# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-arguments
class Dash(object):
    def __init__(
            self,
            name='__main__',
            server=None,
            static_folder='static',
            url_base_pathname='/',
            compress=True,
            **kwargs):

        # pylint-disable: too-many-instance-attributes
        if 'csrf_protect' in kwargs:
            warnings.warn('''
                `csrf_protect` is no longer used,
                CSRF protection has been removed as it is no longer
                necessary.
                See https://github.com/plotly/dash/issues/141 for details.
                ''', DeprecationWarning)

        name = name or 'dash'
        # allow users to supply their own flask server
        self.server = server or Flask(name, static_folder=static_folder)

        self.url_base_pathname = url_base_pathname
        self.config = _AttributeDict({
            'suppress_callback_exceptions': False,
            'routes_pathname_prefix': url_base_pathname,
            'requests_pathname_prefix': url_base_pathname
        })

        # list of dependencies
        self.callback_map = {}

        if compress:
            # gzip
            Compress(self.server)

        @self.server.errorhandler(exceptions.PreventUpdate)
        def _handle_error(error):
            """Handle a halted callback and return an empty 204 response"""
            print(error, file=sys.stderr)
            return ('', 204)

        # static files from the packages
        self.css = Css()
        self.scripts = Scripts()
        self.registered_paths = {}

        # urls

        def add_url(name, view_func, methods=('GET',)):
            self.server.add_url_rule(
                name,
                view_func=view_func,
                endpoint=name,
                methods=list(methods)
            )

        add_url(
            '{}_dash-layout'.format(self.config['routes_pathname_prefix']),
            self.serve_layout)

        add_url(
            '{}_dash-dependencies'.format(
                self.config['routes_pathname_prefix']),
            self.dependencies)

        add_url(
            '{}_dash-update-component'.format(
                self.config['routes_pathname_prefix']),
            self.dispatch,
            ['POST'])

        add_url((
            '{}_dash-component-suites'
            '/<string:package_name>'
            '/<path:path_in_package_dist>').format(
                self.config['routes_pathname_prefix']),
                self.serve_component_suites)

        add_url(
            '{}_dash-routes'.format(self.config['routes_pathname_prefix']),
            self.serve_routes)

        add_url(
            self.config['routes_pathname_prefix'],
            self.index)

        # catch-all for front-end routes
        add_url(
            '{}<path:path>'.format(self.config['routes_pathname_prefix']),
            self.index)

        self.server.before_first_request(self._setup_server)

        self._layout = None
        self._cached_layout = None
        self.routes = []

    @property
    def layout(self):
        return self._layout

    def _layout_value(self):
        if isinstance(self._layout, collections.Callable):
            self._cached_layout = self._layout()
        else:
            self._cached_layout = self._layout
        return self._cached_layout

    @layout.setter
    def layout(self, value):
        if (not isinstance(value, Component) and
                not isinstance(value, collections.Callable)):
            raise Exception(
                ''
                'Layout must be a dash component '
                'or a function that returns '
                'a dash component.')

        self._layout = value

        layout_value = self._layout_value()
        # pylint: disable=protected-access
        self.css._update_layout(layout_value)
        self.scripts._update_layout(layout_value)
        self._collect_and_register_resources(
            self.scripts.get_all_scripts()
        )
        self._collect_and_register_resources(
            self.css.get_all_css()
        )

    def serve_layout(self):
        layout = self._layout_value()

        # TODO - Set browser cache limit - pass hash into frontend
        return flask.Response(
            json.dumps(layout,
                       cls=plotly.utils.PlotlyJSONEncoder),
            mimetype='application/json'
        )

    def _config(self):
        return {
            'url_base_pathname': self.url_base_pathname,
            'requests_pathname_prefix': self.config['requests_pathname_prefix']
        }

    def serve_routes(self):
        return flask.Response(
            json.dumps(self.routes,
                       cls=plotly.utils.PlotlyJSONEncoder),
            mimetype='application/json'
        )

    def _collect_and_register_resources(self, resources):
        # template in the necessary component suite JS bundles
        # add the version number of the package as a query parameter
        # for cache busting
        def _relative_url_path(relative_package_path='', namespace=''):

            # track the registered packages
            if namespace in self.registered_paths:
                self.registered_paths[namespace].append(relative_package_path)
            else:
                self.registered_paths[namespace] = [relative_package_path]

            return '{}_dash-component-suites/{}/{}?v={}'.format(
                self.config['routes_pathname_prefix'],
                namespace,
                relative_package_path,
                importlib.import_module(namespace).__version__
            )

        srcs = []
        for resource in resources:
            if 'relative_package_path' in resource:
                if isinstance(resource['relative_package_path'], str):
                    srcs.append(_relative_url_path(**resource))
                else:
                    for rel_path in resource['relative_package_path']:
                        srcs.append(_relative_url_path(
                            relative_package_path=rel_path,
                            namespace=resource['namespace']
                        ))
            elif 'external_url' in resource:
                if isinstance(resource['external_url'], str):
                    srcs.append(resource['external_url'])
                else:
                    for url in resource['external_url']:
                        srcs.append(url)
            elif 'absolute_path' in resource:
                raise Exception(
                    'Serving files form absolute_path isn\'t supported yet'
                )
        return srcs

    def _generate_css_dist_html(self):
        links = self._collect_and_register_resources(
            self.css.get_all_css()
        )
        return '\n'.join([
            '<link rel="stylesheet" href="{}">'.format(link)
            for link in links
        ])

    def _generate_scripts_html(self):
        # Dash renderer has dependencies like React which need to be rendered
        # before every other script. However, the dash renderer bundle
        # itself needs to be rendered after all of the component's
        # scripts have rendered.
        # The rest of the scripts can just be loaded after React but before
        # dash renderer.
        # pylint: disable=protected-access
        srcs = self._collect_and_register_resources(
            self.scripts._resources._filter_resources(
                dash_renderer._js_dist_dependencies
            ) +
            self.scripts.get_all_scripts() +
            self.scripts._resources._filter_resources(
                dash_renderer._js_dist
            )
        )

        return '\n'.join([
            '<script src="{}"></script>'.format(src)
            for src in srcs
        ])

    def _generate_config_html(self):
        return (
            '<script id="_dash-config" type="application/json">'
            '{}'
            '</script>'
        ).format(json.dumps(self._config()))

    # Serve the JS bundles for each package
    def serve_component_suites(self, package_name, path_in_package_dist):
        if package_name not in self.registered_paths:
            raise Exception(
                'Error loading dependency.\n'
                '"{}" is not a registered library.\n'
                'Registered libraries are: {}'
                .format(package_name, list(self.registered_paths.keys())))

        elif path_in_package_dist not in self.registered_paths[package_name]:
            raise Exception(
                '"{}" is registered but the path requested is not valid.\n'
                'The path requested: "{}"\n'
                'List of registered paths: {}'
                .format(
                    package_name,
                    path_in_package_dist,
                    self.registered_paths
                )
            )

        mimetype = ({
            'js': 'application/JavaScript',
            'css': 'text/css'
        })[path_in_package_dist.split('.')[-1]]
        return Response(
            pkgutil.get_data(package_name, path_in_package_dist),
            mimetype=mimetype
        )

    def index(self, *args, **kwargs):  # pylint: disable=unused-argument
        scripts = self._generate_scripts_html()
        css = self._generate_css_dist_html()
        config = self._generate_config_html()
        title = getattr(self, 'title', 'Dash')
        return '''
        <!DOCTYPE html>
        <html>
            <head>
                <meta charset="UTF-8">
                <title>{}</title>
                {}
            </head>
            <body>
                <div id="react-entry-point">
                    <div class="_dash-loading">
                        Loading...
                    </div>
                </div>
                <footer>
                    {}
                    {}
                </footer>
            </body>
        </html>
        '''.format(title, css, config, scripts)

    def dependencies(self):
        return flask.jsonify([
            {
                'output': {
                    'id': k.split('.')[0],
                    'property': k.split('.')[1]
                },
                'inputs': v['inputs'],
                'state': v['state'],
                'events': v['events']
            } for k, v in list(self.callback_map.items())
        ])

    # pylint: disable=unused-argument, no-self-use
    def react(self, *args, **kwargs):
        raise exceptions.DashException(
            'Yo! `react` is no longer used. \n'
            'Use `callback` instead. `callback` has a new syntax too, '
            'so make sure to call `help(app.callback)` to learn more.')

    def _validate_callback(self, output, inputs, state, events):
        # pylint: disable=too-many-branches
        layout = self._cached_layout or self._layout_value()

        if (layout is None and
                not self.config.first('suppress_callback_exceptions',
                                      'supress_callback_exceptions')):
            # Without a layout, we can't do validation on the IDs and
            # properties of the elements in the callback.
            raise exceptions.LayoutIsNotDefined('''
                Attempting to assign a callback to the application but
                the `layout` property has not been assigned.
                Assign the `layout` property before assigning callbacks.
                Alternatively, suppress this warning by setting
                `app.config['suppress_callback_exceptions']=True`
            '''.replace('    ', ''))

        for args, obj, name in [([output], Output, 'Output'),
                                (inputs, Input, 'Input'),
                                (state, State, 'State'),
                                (events, Event, 'Event')]:

            if not isinstance(args, list):
                raise exceptions.IncorrectTypeException(
                    'The {} argument `{}` is '
                    'not a list of `dash.dependencies.{}`s.'.format(
                        name.lower(), str(args), name
                    ))

            for arg in args:
                if not isinstance(arg, obj):
                    raise exceptions.IncorrectTypeException(
                        'The {} argument `{}` is '
                        'not of type `dash.{}`.'.format(
                            name.lower(), str(arg), name
                        ))

                if (not self.config.first('suppress_callback_exceptions',
                                          'supress_callback_exceptions') and
                        arg.component_id not in layout and
                        arg.component_id != getattr(layout, 'id', None)):
                    raise exceptions.NonExistantIdException('''
                        Attempting to assign a callback to the
                        component with the id "{}" but no
                        components with id "{}" exist in the
                        app\'s layout.\n\n
                        Here is a list of IDs in layout:\n{}\n\n
                        If you are assigning callbacks to components
                        that are generated by other callbacks
                        (and therefore not in the initial layout), then
                        you can suppress this exception by setting
                        `app.config['suppress_callback_exceptions']=True`.
                    '''.format(
                        arg.component_id,
                        arg.component_id,
                        list(layout.keys()) + (
                            [] if not hasattr(layout, 'id') else
                            [layout.id]
                        )
                    ).replace('    ', ''))

                if not self.config.first('suppress_callback_exceptions',
                                         'supress_callback_exceptions'):

                    if getattr(layout, 'id', None) == arg.component_id:
                        component = layout
                    else:
                        component = layout[arg.component_id]

                    if (hasattr(arg, 'component_property') and
                            arg.component_property not in
                            component.available_properties):
                        raise exceptions.NonExistantPropException('''
                            Attempting to assign a callback with
                            the property "{}" but the component
                            "{}" doesn't have "{}" as a property.\n
                            Here is a list of the available properties in "{}":
                            {}
                        '''.format(
                            arg.component_property,
                            arg.component_id,
                            arg.component_property,
                            arg.component_id,
                            component.available_properties).replace(
                                '    ', ''))

                    if (hasattr(arg, 'component_event') and
                            arg.component_event not in
                            component.available_events):
                        raise exceptions.NonExistantEventException('''
                            Attempting to assign a callback with
                            the event "{}" but the component
                            "{}" doesn't have "{}" as an event.\n
                            Here is a list of the available events in "{}":
                            {}
                        '''.format(
                            arg.component_event,
                            arg.component_id,
                            arg.component_event,
                            arg.component_id,
                            component.available_events).replace('    ', ''))

        if state and not events and not inputs:
            raise exceptions.MissingEventsException('''
                This callback has {} `State` {}
                but no `Input` elements or `Event` elements.\n
                Without `Input` or `Event` elements, this callback
                will never get called.\n
                (Subscribing to input components will cause the
                callback to be called whenver their values
                change and subscribing to an event will cause the
                callback to be called whenever the event is fired.)
            '''.format(
                len(state),
                'elements' if len(state) > 1 else 'element'
            ).replace('    ', ''))

        if '.' in output.component_id:
            raise exceptions.IDsCantContainPeriods('''The Output element
            `{}` contains a period in its ID.
            Periods are not allowed in IDs right now.'''.format(
                output.component_id
            ))

        callback_id = '{}.{}'.format(
            output.component_id, output.component_property)
        if callback_id in self.callback_map:
            raise exceptions.CantHaveMultipleOutputs('''
                You have already assigned a callback to the output
                with ID "{}" and property "{}". An output can only have
                a single callback function. Try combining your inputs and
                callback functions together into one function.
            '''.format(
                output.component_id,
                output.component_property).replace('    ', ''))

    # TODO - Update nomenclature.
    # "Parents" and "Children" should refer to the DOM tree
    # and not the dependency tree.
    # The dependency tree should use the nomenclature
    # "observer" and "controller".
    # "observers" listen for changes from their "controllers". For example,
    # if a graph depends on a dropdown, the graph is the "observer" and the
    # dropdown is a "controller". In this case the graph's "dependency" is
    # the dropdown.
    # TODO - Check this map for recursive or other ill-defined non-tree
    # relationships
    # pylint: disable=dangerous-default-value
    def callback(self, output, inputs=[], state=[], events=[]):
        self._validate_callback(output, inputs, state, events)

        callback_id = '{}.{}'.format(
            output.component_id, output.component_property
        )
        self.callback_map[callback_id] = {
            'inputs': [
                {'id': c.component_id, 'property': c.component_property}
                for c in inputs
            ],
            'state': [
                {'id': c.component_id, 'property': c.component_property}
                for c in state
            ],
            'events': [
                {'id': c.component_id, 'event': c.component_event}
                for c in events
            ]
        }

        def wrap_func(func):
            @wraps(func)
            def add_context(*args, **kwargs):

                output_value = func(*args, **kwargs)
                response = {
                    'response': {
                        'props': {
                            output.component_property: output_value
                        }
                    }
                }

                return flask.Response(
                    json.dumps(response,
                               cls=plotly.utils.PlotlyJSONEncoder),
                    mimetype='application/json'
                )

            self.callback_map[callback_id]['callback'] = add_context

            return add_context

        return wrap_func

    def dispatch(self):
        body = flask.request.get_json()
        inputs = body.get('inputs', [])
        state = body.get('state', [])
        output = body['output']

        target_id = '{}.{}'.format(output['id'], output['property'])
        args = []
        for component_registration in self.callback_map[target_id]['inputs']:
            args.append([
                c.get('value', None) for c in inputs if
                c['property'] == component_registration['property'] and
                c['id'] == component_registration['id']
            ][0])

        for component_registration in self.callback_map[target_id]['state']:
            args.append([
                c.get('value', None) for c in state if
                c['property'] == component_registration['property'] and
                c['id'] == component_registration['id']
            ][0])

        return self.callback_map[target_id]['callback'](*args)

    def _setup_server(self):
        self._generate_scripts_html()
        self._generate_css_dist_html()

    def run_server(self,
                   port=8050,
                   debug=False,
                   **flask_run_options):
        self.server.run(port=port, debug=debug, **flask_run_options)


from flask import Flask
from sqlalchemy import create_engine
from werkzeug.utils import find_modules, import_string
from charpy import TEST_DATABASE_URI, DEFAULT_DB_URI


class App(object):

    def __init__(self, config=None, db_engine=None, debug=False):
        """ Initiate the flask app """
        self.app = Flask(__name__)
        self.app.debug = debug

        self.app.config.update(dict(
            DATABASE=TEST_DATABASE_URI,
            DEBUG=debug,
        ))
        self.app.config.update(config or {})
        self.app.config.from_envvar('FLASKR_SETTINGS', silent=True)

        self.register_blueprints()

        if db_engine is None:
            self.app.engine = create_engine(DEFAULT_DB_URI)

    def register_blueprints(self):
        """Automagically register all blueprint named bp in packages

        Check the argument 'bp' in all the modules in the folder inserted in findmodules
        then register all blueprints in the app
        """
        for name in find_modules('charpy.flask.blueprints', recursive=True):
            mod = import_string(name)

            if hasattr(mod, 'bp'):
                self.app.register_blueprint(mod.bp)
        return None


if __name__ == "__main__":  # pragma: no cover
    chartapp = App(debug=True)
    # print(app.blueprints)

    chartapp.app.run()



import unittest
from charpy.flask.factory import create_app
from charpy import chartjs


class TestAppFactory(unittest.TestCase):
    def setUp(self):
        self.app = create_app()
        self.app.testing = True
        self.client = self.app.test_client()

    def test_200_index(self):
        """inital test. ensure flask was set up correctly"""
        response = self.client.get('/', content_type='html/text')
        self.assertEqual(response.status_code, 200)

    def test_201_custom_route(self):
        """ Check that custom route works with the parameter """
        response = self.client.get('/chart_demo/hello/world/', content_type='html/text')
        self.assertTrue(b'hello world!' in response.data)
        self.assertEqual(response.status_code, 200)

    def test_202_error_404_not_found(self):
        """ Check that the 404 is correctly rendered """
        response = self.client.get('/404', content_type='html/text')
        self.assertTrue(b'WAS NOT FOUND' in response.data)
        self.assertEqual(response.status_code, 404)

    def test_203_error_501_not_implemented(self):
        """ Check that the 404 is correctly rendered """
        response = self.client.get('/chart/populationgraph/', content_type='html/text')
        self.assertTrue(b'NOT IMPLEMENTED' in response.data)
        self.assertEqual(response.status_code, 501)

    def test_300_chart_with_static_data(self):
        """ Check that static demo is displayed with value """
        response = self.client.get('/chart_demo/static/', content_type='html/text')
        self.assertTrue(b'#F7464A' in response.data)
        self.assertEqual(response.status_code, 200)

    def test_301_chart_with_input_data(self):
        """inital test. ensure flask was set up correctly"""
        response = self.client.get('/chart_demo/default/', content_type='html/text')
        self.assertTrue(b'pie' in response.data)
        self.assertEqual(response.status_code, 200)

    def test_302_chart_with_chart_object(self):
        """ Check that static demo is displayed with value """
        response = self.client.get('/chart_demo/charpy/', content_type='html/text')
        self.assertTrue(b'bar' in response.data)
        self.assertEqual(response.status_code, 200)

    def case_chart_api_with_type(self, chart_type):
        """ Case for the chart rendered of the good type with the api """
        response = self.client.get('/chart/' + chart_type + '/', content_type='html/text')
        self.assertTrue(chart_type in str(response.data))
        self.assertEqual(response.status_code, 200)

    def test_311_chart_api_with_type_bar(self):
        self.case_chart_api_with_type(chartjs.BAR)

    def test_312_chart_api_with_type_pie(self):
        self.case_chart_api_with_type(chartjs.PIE)

    def test_313_chart_api_with_type_radar(self):
        self.case_chart_api_with_type(chartjs.RADAR)

    def test_314_chart_api_with_type_bubble(self):
        self.case_chart_api_with_type(chartjs.BUBBLE)

    def test_315_chart_api_with_type_scatter(self):
        self.case_chart_api_with_type(chartjs.SCATTER)

    def test_316_chart_api_with_type_polar(self):
        self.case_chart_api_with_type(chartjs.POLAR_AREA)

    def test_400_dataframe_rendered(self):
        """ Check that custom route works with the parameter """
        response = self.client.get('/dataframe/html/', content_type='html/text')
        self.assertTrue(b'<table border="1" class="dataframe">' in response.data)
        self.assertEqual(response.status_code, 200)

    def test_401_get_data_from_dataframe(self):
        """ Check that custom route works with the parameter """
        response = self.client.get('/dataframe/', content_type='application/json')
        self.assertEqual(response.status_code, 200)

    def test_402_get_data_from_dataframe_column(self):
        """ Check that data is reachable """
        response = self.client.get('/dataframe/date/')
        self.assertEqual(response.status_code, 200)

    def test_403_is_data_json(self):
        """ Check that data collected is in json format """
        response = self.client.get('/dataframe/label/')
        self.assertEqual(response.mimetype, 'application/json')

    def test_404_wrong_column_raise_404(self):
        response = self.client.get('/dataframe/not a column name/')
        self.assertEqual(response.status_code, 404)


if __name__ == "__main__":
    unittest.main()



from setuptools import setup, find_packages
from charpy.utils import get_readme_rst

setup(name='charpy',
      version='0.8.0',
      description='Render chart from data',
      long_description=get_readme_rst(),
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.6',
      ],
      keywords='flask chart.js web app chart charpy',
      url='http://github.com/sylhare/charpy',
      author='sylhare',
      author_email='sylhare@outlook.com',
      license='MIT',
      packages=find_packages(exclude=['docs']),
      include_package_data=True,
      install_requires=['flask>=0.12', 'flask-sqlalchemy', 'SQLAlchemy', 'python-dateutil', 'pandas', 'sqlalchemy'],
      tests_require=['pytest==2.9.2'],
      extras_require={
          'dev': ['pypandoc', 'setuptools'],
          'test': ['coverage==4.1']
      },
      zip_safe=False
      )



import os
import re
import pandas as pd
from charpy import DEFAULT_DB_URI
from sqlalchemy import create_engine
from dateutil.parser import parse


def check_path(path):
    """ Check if the path is a directory or a file and return a list of the file in the path """
    list_path = []

    try:
        if os.path.isdir(path):
            list_path = list(map(lambda x: os.path.join(path, x), os.listdir(path)))
        else:
            list_path.append(path)

    except TypeError as error:
        print("Error: Input path not valid - path: '{}' - error: {}".format(path, error))

    return list_path


class Orc(object):
    """
    Wrapper around a pandas dataframe

    Open the file
    Read the data
    Clean it accordingly
    """
    DEFAULT_NAMES = ['date', 'label', 'value']
    DEFAULT_SUB_NAMES = ['article', 'city', 'province']

    def __init__(self, path, sep=None, header=None, names=DEFAULT_NAMES):
        """

        :param path: Where the file(s) are to be opened
        :param sep: if not specified will be found automatically
        :param header: is
            - 'None' there are no header default header are taken
            - An int, then the row[ int ] will be the header and can't be renamed using name
        :param names: names of the columns
        """
        criteria = {'sep': sep, 'header': header}

        if sep is None:
            criteria['engine'] = 'python'
        if header is None:
            criteria['names'] = names

        self.df = pd.DataFrame()
        self.read_csv(path, criteria)

    def read_csv(self, path, criteria):
        """
        Read all of the csv file in the path and add them to the dataframe
        """
        for file in check_path(path):
            if file.endswith('.csv'):
                self.df = self.df.append(pd.read_csv(file, **criteria), ignore_index=True)

    def get_column_values(self):
        """
        :return a list of the values formatted string of each each columns
        """
        str_df = self.df.astype(str)
        return list(map(lambda x: str_df[x].values.tolist(), str_df.columns.values.tolist()))

    # TODO check the formatting is a correct time formatting
    def format_column_date(self, column, formatting="%d/%m/%Y", dayfirst=True):
        """
        Format a column into a date format
        without pd.to_datetime(raw_data['column-name'])

        :param dayfirst: True for the parsing method
        :param formatting: date format to apply
        :param column: column to apply
        """
        c = self.__check_column(column)
        if c:
            try:

                self.df[c] = list(map(lambda x: parse(x, dayfirst=dayfirst).date().strftime(formatting), self.df[c]))
            except (ValueError, TypeError) as error:
                print("Couldn't parse through the dates - {}".format(error))

    def create_from_date_column(self, input_date_column, output_column_name='', formatting='%d/%m/%Y'):
        """
        Create a column with a specific date format from a column where there are dates

        :param input_date_column: source column with date from where you want to create other columns
        :param output_column_name: name of the created column
        :param formatting: date formatting of the output column
                           use regex or simplified notification for day, month or year.
        """

        if output_column_name == '':
            output_column_name = formatting.replace("%", "")

        self.df[output_column_name] = self.df[self.__check_column(input_date_column)]

        if formatting in ("month", "Month", "m", "M"):
            formatting = "%B"
        elif formatting in ("day", "Day", "d", "D"):
            formatting = "%A"
        elif formatting in ("year", "Year", "y", "Y"):
            formatting = "%Y"

        self.format_column_date(output_column_name, formatting=formatting)

    def format_column_list(self, column, regex=r'  +'):
        """
        Format the column into a list using regex

        :param column: column to apply
        :param regex: by default more than two spaces
        :return:
        """

        c = self.__check_column(column)
        if c:
            self.df[c] = list(map(lambda x: re.split(regex, x), self.df[c]))

    def create_from_list_column(self, column, names=DEFAULT_SUB_NAMES):
        """
        Take a column that has list values and create columns for it

        Add a buffer to the list in case there are not enough elements to create the column
        It will add '' instead

        :return:
        """

        c = self.__check_column(column)
        if c:
            empty_buffer = ['' for i in names]

            try:
                for r in range(len(self.df[c])):
                    if type(self.df.at[r, c]) == list:
                        self.df.at[r, c].extend(empty_buffer)
                    else:
                        raise TypeError('Error: The column needs to only contain list objects')

                for i in range(len(names)):
                    self.df[names[i]] = map(lambda x: x[i], self.df[c])

            except TypeError as error:
                print(error)

    def __check_column(self, column):
        """
        Column can be a number of a name

        :param column:
        :return: the column name
        """
        try:
            if int == type(column):
                if column >= 0:
                    c_name = self.df.columns[column]
                else:
                    raise TypeError("Error: column should be a positive number")

            elif column in self.df:
                c_name = column
            else:
                raise ValueError("Error: column should be the number or the name of the column")

        except (TypeError, ValueError, IndexError) as error:
            print(error)
            c_name = False

        return c_name

    def get_json_data(self, column_name):
        """ Return the json data of a column """
        return self.df[column_name].to_json()

    # TODO change 'replace' by fail and create a backup if enable before retrying with replace
    def create_sql_db(self, name="charpy", db_path=DEFAULT_DB_URI):
        """
        Dump the information of the dataframe into an sql format
        If the sql database has already the same name table it will replace it.

        :param db_path: Path of the database
        :param name - will be the name of the sql table
        """
        engine = create_engine('sqlite:///' + db_path)
        self.df.to_sql(name, engine, if_exists='replace')


if __name__ == "__main__":  # pragma: no cover
    from charpy import MOCK_PATH, ROOT_PATH
    s = Orc(os.path.join(MOCK_PATH, 'pcbanking.csv'))
    # print(Orc(os.path.join(DATA_PATH, "demo.csv")).df)
    s.df['value'] = s.df['value'].astype('float64')
    # print(s.df['value'].dtype)
    # print(os.listdir(DATA_PATH))
    s.format_column_date('date')
    # s.format_column_list('label')
    # s.create_from_list_column('label')
    # print(s.df.to_html())
    s.create_sql_db()






import os
from charpy import README_RST_PATH, README_MD_PATH
from charpy import ROOT_PATH


def readme_md_to_rst():
    """
    Use pypandoc to create a .rst version of the .md README

    :return: a string with the rst
    """
    try:
        import pypandoc
        result = pypandoc.convert_file(README_MD_PATH, 'rst')

    except(IOError, ImportError):
        result = open(README_MD_PATH).read()

    return result


def create_readme_rst():
    """
    create the readme rst
    """
    result = readme_md_to_rst()
    with open(README_RST_PATH, 'w') as f:
        f.write(result)


def get_readme_rst():
    """
    Read the README.rst file and return a string with it content
    :return: README.rst in a string
    """
    readme_path = os.path.join(ROOT_PATH, "README.rst")
    with open(readme_path, 'r') as f:
        readme = f.read()

    return readme


if __name__ == "__main__":  # pragma: no cover
    create_readme_rst()
    # print(get_readme_rst())


import os

ROOT_PATH = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
DATA_PATH = os.path.join(ROOT_PATH, "data")
MOCK_PATH = os.path.join(DATA_PATH, "mock")
SQL_PATH = os.path.join(DATA_PATH, "sql")

TEST_DATABASE_URI = os.path.join(SQL_PATH, "test_database.db")
DEFAULT_DB_URI = os.path.join(SQL_PATH, "charpy.db")

README_RST_PATH = os.path.join(ROOT_PATH, "README.rst")
README_MD_PATH = os.path.join(ROOT_PATH, os.path.join("docs", "README.md"))
